/*
** Network Audio Front-End (netaudio)
** Receive data routines
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <termios.h>
#include <unistd.h>
#include <netdb.h>

#include "netaudio.h"

static int recv_socket;

void open_receive(void)
{
    struct protoent *p;
    struct sockaddr_in sin;

    bzero((char *) &sin, sizeof(sin));
    sin.sin_family = AF_INET;

    if (!strcmp(bind_address, "127.0.0.1"))
	sin.sin_addr.s_addr = INADDR_ANY;
    else
	sin.sin_addr.s_addr = inet_addr(bind_address);

    if ((sin.sin_port = htons(bind_port)) == 0) {
	fprintf(stderr, "netaudio(receive): cannot get bind port for %d\n", bind_port);
	exit(-1);
    }

    if ((p = getprotobyname("tcp")) == 0) {
	perror("getprotobyname(receive)");
	exit(-1);
    }

    if ((recv_socket = socket(AF_INET, SOCK_STREAM, p->p_proto)) < 0) {
	perror("socket(receive)");
	exit(-1);
    }

    if (bind(recv_socket, (struct sockaddr *) &sin, sizeof(sin)) < 0) {
	perror("bind(receive)");
	exit(-1);
    }

    if (listen(recv_socket, 5) < 0) {
	perror("listen(receive)");
	exit(-1);
    }

    if (!is_quiet)
	fprintf(stderr, "netaudio: receiver ready for one connection.\n");
}

void receive_connect(void)
{
    struct protoent *p;
    struct sockaddr_in sin;
    int sock, num_read, num_write;

    if (strcmp(bind_address, "127.0.0.1")) {
	if (!is_quiet)
	    fprintf(stderr, "netaudio(receive): bind address ignored during connect.\n");
    }

    bzero((char *) &sin, sizeof(sin));
    sin.sin_family = AF_INET;
    sin.sin_port = htons(port);
    sin.sin_addr.s_addr = inet_addr(address);

    p = getprotobyname("tcp");
    if ((sock = socket(AF_INET, SOCK_STREAM, p->p_proto)) < 0) {
	perror("socket(receive)");
	exit(-1);
    }

    if (connect(sock, (struct sockaddr *) &sin, sizeof(sin)) < 0) {
	perror("connect(receive)");
	exit(-1);
    }

    tcdrain(sock);

    if (!is_quiet)
	fprintf(stderr, "netaudio(receive): remote connection established.\n");

    while(1) {
	char buffer[MAX_BUFFER];

	if ((num_read = read(sock, &buffer, frame_size)) <= 0) {
	    if (num_read)
		perror("read(receive)");

	    if (!is_quiet)
		fprintf(stderr, "netaudio(receive): read error on remote end (premature close?)\n");

	    (void) shutdown(sock, 2);
	    close(sock);

	    exit(-1);
	}

	if ((num_write = write(STDOUT_FILENO, (char *) &buffer, num_read)) <= 0) {
	    if (num_write)
		perror("write(receive)");

	    if (!is_quiet)
		fprintf(stderr, "netaudio(receive): connection ended.\n");

	    (void) shutdown(sock, 2);
	    close(sock);

	    exit(-1);
	}

	if (is_http)
	    fflush(stdout);
    }
}

void receive_loop(void)
{
    int fds = getdtablesize(), sock = 0;
    struct sockaddr_in sin;
    fd_set activeset, readset;

    open_receive();

    FD_ZERO(&activeset);
    FD_SET(recv_socket, &activeset);

    while(1) {
	struct timeval t;

	t.tv_sec = 100;
	t.tv_usec = 0;

	bcopy(&activeset, &readset, sizeof(readset));

	if (select(fds, &readset, NULL, NULL, &t) < 0)
	    perror("select(receive)");

	if (FD_ISSET(recv_socket, &readset)) {
	    int addrlen, newsock;

	    addrlen = sizeof(struct sockaddr_in);
	    newsock = accept(recv_socket, (struct sockaddr *) &sin, &addrlen);

	    if (newsock < 0)
		perror("accept");
	    else {
		if (sock) {
		    if (!is_quiet)
			fprintf(stderr, "netaudio(receive): unauthorized connect from %s\n", inet_ntoa(sin.sin_addr));

		    (void) shutdown(newsock, 2);
		    close(newsock);
		} else {
		    sock = newsock;
		    FD_SET(sock, &activeset);

		    if (!is_quiet)
			fprintf(stderr, "netaudio(receive): connect from %s\n", inet_ntoa(sin.sin_addr));
		}
	    }
	}

	if (sock) {
	    if (FD_ISSET(sock, &readset)) {
		char buffer[MAX_BUFFER];
		int num_read;

		if ((num_read = read(sock, &buffer, frame_size)) <= 0) {
		    if (!is_quiet)
			fprintf(stderr, "netaudio(recieve): connection lost.\n");

		    if (num_read < 0)
			perror("read(receive)");

		    (void) shutdown(sock, 2);
		    close(sock);

		    exit(0);
		} else {
		    if (write(STDOUT_FILENO, (char *) &buffer, num_read) <= 0) {
			perror("write(receive)");

			(void) shutdown(sock, 2);
			close(sock);

			if (!is_quiet)
			    fprintf(stderr, "netaudio(receive): shutdown.\n");

			exit(0);
		    }

		    if (is_http)
			fflush(stdout);
		}
	    }
	}
    }
}
